@@ -23,7 +23,6 @@
 #include "color.h"
 #include "config.h"
 #include "cppcheck.h"
-#include "cppcheckexecutor.h"
 #include "errorlogger.h"
 #include "errortypes.h"
 #include "filelister.h"
@@ -42,7 +41,6 @@
 #include <algorithm>
 #include <cassert>
 #include <climits>
-#include <cstdint>
 #include <cstdio>
 #include <cstdlib> // EXIT_FAILURE
 #include <cstring>
@@ -164,8 +162,8 @@ bool CmdLineParser::fillSettingsFromArgs(int argc, const char* const argv[])
 
     // Check that all include paths exist
     {
-        for (std::list<std::string>::iterator iter = mSettings.includePaths.begin();
-             iter != mSettings.includePaths.end();
+        for (std::list<std::string>::const_iterator iter = mSettings.includePaths.cbegin();
+             iter != mSettings.includePaths.cend();
              ) {
             const std::string path(Path::toNativeSeparators(*iter));
             if (Path::isDirectory(path))
@@ -342,9 +340,9 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                 return Result::Fail;
             {
                 XMLErrorMessagesLogger xmlLogger;
-                std::cout << ErrorMessage::getXMLHeader(mSettings.cppcheckCfgProductName);
+                std::cout << ErrorMessage::getXMLHeader(mSettings.cppcheckCfgProductName, 2);
                 CppCheck::getErrorMessages(xmlLogger);
-                std::cout << ErrorMessage::getXMLFooter() << std::endl;
+                std::cout << ErrorMessage::getXMLFooter(2) << std::endl;
             }
             return Result::Exit;
         }
@@ -370,7 +368,6 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
     ImportProject project;
 
     bool executorAuto = true;
-    int8_t logMissingInclude{0};
 
     for (int i = 1; i < argc; i++) {
         if (argv[i][0] == '-') {
@@ -593,9 +590,6 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                     mLogger.printError(errmsg);
                     return Result::Fail;
                 }
-                if (std::string(argv[i] + 10).find("missingInclude") != std::string::npos) {
-                    --logMissingInclude;
-                }
             }
 
             // dump cppcheck data
@@ -615,13 +609,6 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                     mSettings.addEnabled("performance");
                     mSettings.addEnabled("portability");
                 }
-                if (enable_arg.find("information") != std::string::npos && logMissingInclude == 0) {
-                    ++logMissingInclude;
-                    mSettings.addEnabled("missingInclude");
-                }
-                if (enable_arg.find("missingInclude") != std::string::npos) {
-                    --logMissingInclude;
-                }
             }
 
             // --error-exitcode=1
@@ -649,7 +636,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                     return Result::Fail;
                 }
                 mSettings.exceptionHandling = true;
-                CppCheckExecutor::setExceptionOutput((exceptionOutfilename == "stderr") ? stderr : stdout);
+                mSettings.exceptionOutput = (exceptionOutfilename == "stderr") ? stderr : stdout;
 #else
                 mLogger.printError("Option --exception-handling is not supported since Cppcheck has not been built with any exception handling enabled.");
                 return Result::Fail;
@@ -757,7 +744,6 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 
                 if (!path.empty()) {
                     path = Path::removeQuotationMarks(std::move(path));
-                    path = Path::fromNativeSeparators(std::move(path));
                     path = Path::simplifyPath(std::move(path));
 
                     // TODO: this only works when it exists
@@ -917,8 +903,14 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 
             // max ctu depth
             else if (std::strncmp(argv[i], "--max-ctu-depth=", 16) == 0) {
-                if (!parseNumberArg(argv[i], 16, mSettings.maxCtuDepth))
+                int temp = 0;
+                if (!parseNumberArg(argv[i], 16, temp))
                     return Result::Fail;
+                if (temp > 10) {
+                    mLogger.printMessage("--max-ctu-depth is being capped at 10. This limitation will be removed in a future Cppcheck version.");
+                    temp = 10;
+                }
+                mSettings.maxCtuDepth = temp;
             }
 
             else if (std::strcmp(argv[i], "--no-cpp-header-probe") == 0) {
@@ -929,6 +921,20 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
             else if (std::strncmp(argv[i], "--output-file=", 14) == 0)
                 mSettings.outputFile = Path::simplifyPath(argv[i] + 14);
 
+            else if (std::strncmp(argv[i], "--output-format=", 16) == 0) {
+                const std::string format = argv[i] + 16;
+                if (format == "sarif")
+                    mSettings.outputFormat = Settings::OutputFormat::sarif;
+                else if (format == "xml")
+                    mSettings.outputFormat = Settings::OutputFormat::xml;
+                else {
+                    mLogger.printError("argument to '--output-format=' must be 'sarif' or 'xml'.");
+                    return Result::Fail;
+                }
+                mSettings.xml = (mSettings.outputFormat == Settings::OutputFormat::xml);
+            }
+
+
             // Experimental: limit execution time for extended valueflow analysis. basic valueflow analysis
             // is always executed.
             else if (std::strncmp(argv[i], "--performance-valueflow-max-time=", 33) == 0) {
@@ -963,6 +969,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 
             // Write results in results.plist
             else if (std::strncmp(argv[i], "--plist-output=", 15) == 0) {
+                mSettings.outputFormat = Settings::OutputFormat::plist;
                 mSettings.plistOutput = Path::simplifyPath(argv[i] + 15);
                 if (mSettings.plistOutput.empty())
                     mSettings.plistOutput = ".";
@@ -981,7 +988,8 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
             }
 
             // Special Cppcheck Premium options
-            else if (std::strncmp(argv[i], "--premium=", 10) == 0 && isCppcheckPremium()) {
+            else if ((std::strncmp(argv[i], "--premium=", 10) == 0 || std::strncmp(argv[i], "--premium-", 10) == 0) && isCppcheckPremium()) {
+                // valid options --premium=..
                 const std::set<std::string> valid{
                     "autosar",
                     "cert-c-2016",
@@ -995,6 +1003,11 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                     "misra-cpp-2023",
                     "bughunting",
                     "safety"};
+                // valid options --premium-..=
+                const std::set<std::string> valid2{
+                    "cert-c-int-precision",
+                    "license-file"
+                };
 
                 if (std::strcmp(argv[i], "--premium=safety-off") == 0) {
                     mSettings.safety = false;
@@ -1005,8 +1018,9 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                 if (!mSettings.premiumArgs.empty())
                     mSettings.premiumArgs += " ";
                 const std::string p(argv[i] + 10);
-                if (!valid.count(p) && !startsWith(p, "cert-c-int-precision=")) {
-                    mLogger.printError("invalid --premium option '" + p + "'.");
+                const std::string p2(p.find('=') != std::string::npos ? p.substr(0, p.find('=')) : "");
+                if (!valid.count(p) && !valid2.count(p2)) {
+                    mLogger.printError("invalid --premium option '" + (p2.empty() ? p : p2) + "'.");
                     return Result::Fail;
                 }
                 mSettings.premiumArgs += "--" + p;
@@ -1157,33 +1171,35 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                         Settings::Rule rule;
 
                         for (const tinyxml2::XMLElement *subnode = node->FirstChildElement(); subnode; subnode = subnode->NextSiblingElement()) {
+                            const char * const subname = subnode->Name();
                             const char * const subtext = subnode->GetText();
-                            if (std::strcmp(subnode->Name(), "tokenlist") == 0) {
+                            if (std::strcmp(subname, "tokenlist") == 0) {
                                 rule.tokenlist = empty_if_null(subtext);
                             }
-                            else if (std::strcmp(subnode->Name(), "pattern") == 0) {
+                            else if (std::strcmp(subname, "pattern") == 0) {
                                 rule.pattern = empty_if_null(subtext);
                             }
-                            else if (std::strcmp(subnode->Name(), "message") == 0) {
+                            else if (std::strcmp(subname, "message") == 0) {
                                 for (const tinyxml2::XMLElement *msgnode = subnode->FirstChildElement(); msgnode; msgnode = msgnode->NextSiblingElement()) {
+                                    const char * const msgname = msgnode->Name();
                                     const char * const msgtext = msgnode->GetText();
-                                    if (std::strcmp(msgnode->Name(), "severity") == 0) {
+                                    if (std::strcmp(msgname, "severity") == 0) {
                                         rule.severity = severityFromString(empty_if_null(msgtext));
                                     }
-                                    else if (std::strcmp(msgnode->Name(), "id") == 0) {
+                                    else if (std::strcmp(msgname, "id") == 0) {
                                         rule.id = empty_if_null(msgtext);
                                     }
-                                    else if (std::strcmp(msgnode->Name(), "summary") == 0) {
+                                    else if (std::strcmp(msgname, "summary") == 0) {
                                         rule.summary = empty_if_null(msgtext);
                                     }
                                     else {
-                                        mLogger.printError("unable to load rule-file '" + ruleFile + "' - unknown element '" + msgnode->Name() + "' encountered in 'message'.");
+                                        mLogger.printError("unable to load rule-file '" + ruleFile + "' - unknown element '" + msgname + "' encountered in 'message'.");
                                         return Result::Fail;
                                     }
                                 }
                             }
                             else {
-                                mLogger.printError("unable to load rule-file '" + ruleFile + "' - unknown element '" + subnode->Name() + "' encountered in 'rule'.");
+                                mLogger.printError("unable to load rule-file '" + ruleFile + "' - unknown element '" + subname + "' encountered in 'rule'.");
                                 return Result::Fail;
                             }
                         }
@@ -1261,12 +1277,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
             // --std
             else if (std::strncmp(argv[i], "--std=", 6) == 0) {
                 const std::string std = argv[i] + 6;
-                Standards tmp;
-                if (tmp.setC(std)) {
-                    mSettings.standards.c = tmp.c;
-                } else if (tmp.setCPP(std)) {
-                    mSettings.standards.cpp = tmp.cpp;
-                } else {
+                if (!mSettings.standards.setStd(std)) {
                     mLogger.printError("unknown --std value '" + std + "'");
                     return Result::Fail;
                 }
@@ -1371,23 +1382,26 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                 mSettings.verbose = true;
 
             // Write results in results.xml
-            else if (std::strcmp(argv[i], "--xml") == 0)
+            else if (std::strcmp(argv[i], "--xml") == 0) {
                 mSettings.xml = true;
+                mSettings.outputFormat = Settings::OutputFormat::xml;
+            }
 
             // Define the XML file version (and enable XML output)
             else if (std::strncmp(argv[i], "--xml-version=", 14) == 0) {
                 int tmp;
                 if (!parseNumberArg(argv[i], 14, tmp))
                     return Result::Fail;
-                if (tmp != 2) {
-                    // We only have xml version 2
-                    mLogger.printError("'--xml-version' can only be 2.");
+                if (tmp != 2 && tmp != 3) {
+                    // We only have xml version 2 and 3
+                    mLogger.printError("'--xml-version' can only be 2 or 3.");
                     return Result::Fail;
                 }
 
                 mSettings.xml_version = tmp;
                 // Enable also XML if version is set
                 mSettings.xml = true;
+                mSettings.outputFormat = Settings::OutputFormat::xml;
             }
 
             else {
@@ -1404,9 +1418,6 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
         }
     }
 
-    if (logMissingInclude == 1)
-        mLogger.printMessage("'--enable=information' will no longer implicitly enable 'missingInclude' starting with 2.16. Please enable it explicitly if you require it.");
-
     if (!loadCppcheckCfg())
         return Result::Fail;
 
@@ -1636,6 +1647,10 @@ void CmdLineParser::printHelp() const
         "                         is 2. A larger value will mean more errors can be found\n"
         "                         but also means the analysis will be slower.\n"
         "    --output-file=<file> Write results to file, rather than standard error.\n"
+        "    --output-format=<format>\n"
+        "                        Specify the output format. The available formats are:\n"
+        "                          * sarif\n"
+        "                          * xml\n"
         "    --platform=<type>, --platform=<file>\n"
         "                         Specifies platform specific types and sizes. The\n"
         "                         available builtin platforms are:\n"
