@@ -50,7 +50,7 @@
 #include <ctime>
 #include <exception> // IWYU pragma: keep
 #include <fstream>
-#include <iostream> // <- TEMPORARY
+#include <iostream>
 #include <new>
 #include <set>
 #include <sstream>
@@ -139,7 +139,7 @@ static std::string getDumpFileName(const Settings& settings, const std::string&
         return settings.dumpFile;
 
     std::string extension;
-    if (settings.dump)
+    if (settings.dump || !settings.buildDir.empty())
         extension = ".dump";
     else
         extension = "." + std::to_string(settings.pid) + ".dump";
@@ -167,7 +167,7 @@ static void createDumpFile(const Settings& settings,
     if (!fdump.is_open())
         return;
 
-    {
+    if (!settings.buildDir.empty()) {
         std::ofstream fout(getCtuInfoFileName(dumpFile));
     }
 
@@ -430,8 +430,11 @@ unsigned int CppCheck::checkClang(const FileWithDetails &file)
         mErrorLogger.reportOut(std::string("Checking ") + file.spath() + " ...", Color::FgGreen);
 
     // TODO: get language from FileWithDetails object
-    // TODO: this ignores the configured language
-    const bool isCpp = Path::identify(file.spath(), mSettings.cppHeaderProbe) == Standards::Language::CPP;
+    bool isCpp;
+    if (mSettings.enforcedLang != Standards::None)
+        isCpp = (mSettings.enforcedLang == Standards::CPP);
+    else
+        isCpp = Path::identify(file.spath(), mSettings.cppHeaderProbe) == Standards::Language::CPP;
     const std::string langOpt = isCpp ? "-x c++" : "-x c";
     const std::string analyzerInfo = mSettings.buildDir.empty() ? std::string() : AnalyzerInformation::getAnalyzerInfoFile(mSettings.buildDir, file.spath(), emptyString);
     const std::string clangcmd = analyzerInfo + ".clang-cmd";
@@ -446,9 +449,10 @@ unsigned int CppCheck::checkClang(const FileWithDetails &file)
 #endif
 
     std::string flags(langOpt + " ");
-    // TODO: does not apply C standard
-    if (isCpp && !mSettings.standards.stdValue.empty())
-        flags += "-std=" + mSettings.standards.stdValue + " ";
+    if (isCpp && !mSettings.standards.stdValueCPP.empty())
+        flags += "-std=" + mSettings.standards.stdValueCPP + " ";
+    if (!isCpp && !mSettings.standards.stdValueC.empty())
+        flags += "-std=" + mSettings.standards.stdValueC + " ";
 
     for (const std::string &i: mSettings.includePaths)
         flags += "-I" + i + " ";
@@ -512,7 +516,7 @@ unsigned int CppCheck::checkClang(const FileWithDetails &file)
                              mSettings,
                              &s_timerResults);
         if (mSettings.debugnormal)
-            tokenizer.printDebugOutput(1);
+            tokenizer.printDebugOutput(1, std::cout);
         checkNormalTokens(tokenizer);
 
         // create dumpfile
@@ -731,8 +735,7 @@ unsigned int CppCheck::checkFile(const FileWithDetails& file, const std::string
             mSettings.supprs.nomsg.dump(oss);
             dumpProlog += oss.str();
         }
-        tokens1.removeComments();
-        preprocessor.removeComments();
+        preprocessor.removeComments(tokens1);
 
         if (!mSettings.buildDir.empty()) {
             // Get toolinfo
@@ -772,9 +775,9 @@ unsigned int CppCheck::checkFile(const FileWithDetails& file, const std::string
 
         // Get directives
         std::list<Directive> directives = preprocessor.createDirectives(tokens1);
-        preprocessor.simplifyPragmaAsm(&tokens1);
+        preprocessor.simplifyPragmaAsm(tokens1);
 
-        preprocessor.setPlatformInfo(&tokens1);
+        Preprocessor::setPlatformInfo(tokens1, mSettings);
 
         // Get configurations..
         std::set<std::string> configurations;
@@ -1411,12 +1414,13 @@ void CppCheck::executeAddons(const std::vector<std::string>& files, const std::s
     if (mSettings.addons.empty() || files.empty())
         return;
 
-    FilesDeleter filesDeleter;
+    const bool isCtuInfo = endsWith(files[0], ".ctu-info");
 
+    FilesDeleter filesDeleter;
     std::string fileList;
 
-    if (files.size() >= 2 || endsWith(files[0], ".ctu-info")) {
-        fileList = Path::getPathFromFilename(files[0]) + FILELIST + std::to_string(mSettings.pid);
+    if (files.size() >= 2) {
+        fileList = Path::getPathFromFilename(files[0]) + FILELIST + ("-" + std::to_string(mSettings.pid)) + ".txt";
         filesDeleter.addFile(fileList);
         std::ofstream fout(fileList);
         for (const std::string& f: files)
@@ -1426,8 +1430,10 @@ void CppCheck::executeAddons(const std::vector<std::string>& files, const std::s
     // ensure all addons have already been resolved - TODO: remove when settings are const after creation
     assert(mSettings.addonInfos.size() == mSettings.addons.size());
 
+    std::string ctuInfo;
+
     for (const AddonInfo &addonInfo : mSettings.addonInfos) {
-        if (addonInfo.name != "misra" && !addonInfo.ctu && endsWith(files.back(), ".ctu-info"))
+        if (isCtuInfo && addonInfo.name != "misra" && !addonInfo.ctu)
             continue;
 
         const std::vector<picojson::value> results =
@@ -1442,6 +1448,18 @@ void CppCheck::executeAddons(const std::vector<std::string>& files, const std::s
 
             ErrorMessage errmsg;
 
+            if (obj.count("summary") > 0) {
+                if (!mSettings.buildDir.empty()) {
+                    ctuInfo += res.serialize() + "\n";
+                } else {
+                    errmsg.severity = Severity::internal;
+                    errmsg.id = "ctuinfo";
+                    errmsg.setmsg(res.serialize());
+                    reportErr(errmsg);
+                }
+                continue;
+            }
+
             if (obj.count("file") > 0) {
                 std::string fileName = obj["file"].get<std::string>();
                 const int64_t lineNumber = obj["linenr"].get<int64_t>();
@@ -1480,13 +1498,30 @@ void CppCheck::executeAddons(const std::vector<std::string>& files, const std::s
             reportErr(errmsg);
         }
     }
+
+    if (!mSettings.buildDir.empty() && !isCtuInfo) {
+        const std::string& ctuInfoFile = getCtuInfoFileName(files[0]);
+        std::ofstream fout(ctuInfoFile);
+        fout << ctuInfo;
+    }
 }
 
-void CppCheck::executeAddonsWholeProgram(const std::list<FileWithDetails> &files, const std::list<FileSettings>& fileSettings)
+void CppCheck::executeAddonsWholeProgram(const std::list<FileWithDetails> &files, const std::list<FileSettings>& fileSettings, const std::string& ctuInfo)
 {
     if (mSettings.addons.empty())
         return;
 
+    if (mSettings.buildDir.empty()) {
+        const std::string fileName = std::to_string(mSettings.pid) + ".ctu-info";
+        FilesDeleter filesDeleter;
+        filesDeleter.addFile(fileName);
+        std::ofstream fout(fileName);
+        fout << ctuInfo;
+        fout.close();
+        executeAddons({fileName}, "");
+        return;
+    }
+
     std::vector<std::string> ctuInfoFiles;
     for (const auto &f: files) {
         const std::string &dumpFileName = getDumpFileName(mSettings, f.path());
@@ -1504,11 +1539,6 @@ void CppCheck::executeAddonsWholeProgram(const std::list<FileWithDetails> &files
         const ErrorMessage errmsg = ErrorMessage::fromInternalError(e, nullptr, "", "Bailing out from analysis: Whole program analysis failed");
         reportErr(errmsg);
     }
-
-    if (mSettings.buildDir.empty()) {
-        for (const std::string &f: ctuInfoFiles)
-            std::remove(f.c_str());
-    }
 }
 
 Settings &CppCheck::settings()
@@ -1679,11 +1709,7 @@ void CppCheck::reportProgress(const std::string &filename, const char stage[], c
 void CppCheck::getErrorMessages(ErrorLogger &errorlogger)
 {
     Settings s;
-    s.severity.enable(Severity::warning);
-    s.severity.enable(Severity::style);
-    s.severity.enable(Severity::portability);
-    s.severity.enable(Severity::performance);
-    s.severity.enable(Severity::information);
+    s.addEnabled("all");
 
     CppCheck cppcheck(errorlogger, true, nullptr);
     cppcheck.purgedConfigurationMessage(emptyString,emptyString);
@@ -1779,8 +1805,6 @@ void CppCheck::analyseClangTidy(const FileSettings &fileSettings)
 bool CppCheck::analyseWholeProgram()
 {
     bool errors = false;
-    // Init CTU
-    CTU::maxCtuDepth = mSettings.maxCtuDepth;
     // Analyse the tokens
     CTU::FileInfo ctu;
     if (mSettings.useSingleJob() || !mSettings.buildDir.empty())
@@ -1804,13 +1828,9 @@ bool CppCheck::analyseWholeProgram()
     return errors && (mExitCode > 0);
 }
 
-unsigned int CppCheck::analyseWholeProgram(const std::string &buildDir, const std::list<FileWithDetails> &files, const std::list<FileSettings>& fileSettings)
+unsigned int CppCheck::analyseWholeProgram(const std::string &buildDir, const std::list<FileWithDetails> &files, const std::list<FileSettings>& fileSettings, const std::string& ctuInfo)
 {
-    executeAddonsWholeProgram(files, fileSettings);
-    if (buildDir.empty()) {
-        removeCtuInfoFiles(files, fileSettings);
-        return mExitCode;
-    }
+    executeAddonsWholeProgram(files, fileSettings, ctuInfo);
     if (mSettings.checks.isEnabled(Checks::unusedFunction))
         CheckUnusedFunctions::analyseWholeProgram(mSettings, *this, buildDir);
     std::list<Check::FileInfo*> fileInfoList;
@@ -1857,9 +1877,6 @@ unsigned int CppCheck::analyseWholeProgram(const std::string &buildDir, const st
         }
     }
 
-    // Set CTU max depth
-    CTU::maxCtuDepth = mSettings.maxCtuDepth;
-
     // Analyse the tokens
     // cppcheck-suppress shadowFunction - TODO: fix this
     for (Check *check : Check::instances())
@@ -1874,22 +1891,6 @@ unsigned int CppCheck::analyseWholeProgram(const std::string &buildDir, const st
     return mExitCode;
 }
 
-void CppCheck::removeCtuInfoFiles(const std::list<FileWithDetails> &files, const std::list<FileSettings>& fileSettings)
-{
-    if (mSettings.buildDir.empty()) {
-        for (const auto& f: files) {
-            const std::string &dumpFileName = getDumpFileName(mSettings, f.path());
-            const std::string &ctuInfoFileName = getCtuInfoFileName(dumpFileName);
-            std::remove(ctuInfoFileName.c_str());
-        }
-        for (const auto& fs: fileSettings) {
-            const std::string &dumpFileName = getDumpFileName(mSettings, fs.filename());
-            const std::string &ctuInfoFileName = getCtuInfoFileName(dumpFileName);
-            std::remove(ctuInfoFileName.c_str());
-        }
-    }
-}
-
 // cppcheck-suppress unusedFunction - only used in tests
 void CppCheck::resetTimerResults()
 {
