@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,8 +32,8 @@
 #include <algorithm>
 #include <array>
 #include <cstddef>
-#include <iterator> // back_inserter
-#include <sstream> // IWYU pragma: keep
+#include <iterator>
+#include <sstream>
 #include <utility>
 
 #include <simplecpp.h>
@@ -43,29 +43,27 @@ static bool sameline(const simplecpp::Token *tok1, const simplecpp::Token *tok2)
     return tok1 && tok2 && tok1->location.sameline(tok2->location);
 }
 
-/**
- * Remove heading and trailing whitespaces from the input parameter.
- * If string is all spaces/tabs, return empty string.
- * @param s The string to trim.
- */
-static std::string trim(const std::string& s)
-{
-    const std::string::size_type beg = s.find_first_not_of(" \t");
-    if (beg == std::string::npos)
-        return "";
-    const std::string::size_type end = s.find_last_not_of(" \t");
-    return s.substr(beg, end - beg + 1);
-}
+Directive::Directive(const simplecpp::Location & _loc, std::string _str) :
+    file(_loc.file()),
+    linenr(_loc.line),
+    str(std::move(_str))
+{}
 
-Directive::Directive(std::string _file, const int _linenr, const std::string &_str) :
+Directive::Directive(std::string _file, const int _linenr, std::string _str) :
     file(std::move(_file)),
     linenr(_linenr),
-    str(trim(_str))
+    str(std::move(_str))
+{}
+
+Directive::DirectiveToken::DirectiveToken(const simplecpp::Token & _tok) :
+    line(_tok.location.line),
+    column(_tok.location.col),
+    tokStr(_tok.str())
 {}
 
 char Preprocessor::macroChar = char(1);
 
-Preprocessor::Preprocessor(const Settings& settings, ErrorLogger *errorLogger) : mSettings(settings), mErrorLogger(errorLogger)
+Preprocessor::Preprocessor(const Settings& settings, ErrorLogger &errorLogger) : mSettings(settings), mErrorLogger(errorLogger)
 {}
 
 Preprocessor::~Preprocessor()
@@ -83,7 +81,7 @@ namespace {
     };
 }
 
-static bool parseInlineSuppressionCommentToken(const simplecpp::Token *tok, std::list<Suppressions::Suppression> &inlineSuppressions, std::list<BadInlineSuppression> &bad)
+static bool parseInlineSuppressionCommentToken(const simplecpp::Token *tok, std::list<SuppressionList::Suppression> &inlineSuppressions, std::list<BadInlineSuppression> &bad)
 {
     const std::string cppchecksuppress("cppcheck-suppress");
 
@@ -106,7 +104,7 @@ static bool parseInlineSuppressionCommentToken(const simplecpp::Token *tok, std:
     if (pos2 == std::string::npos)
         return false;
 
-    Suppressions::Type errorType = Suppressions::Type::unique;
+    SuppressionList::Type errorType = SuppressionList::Type::unique;
 
     // determine prefix if specified
     if (posEndComment >= (pos1 + cppchecksuppress.size() + 1)) {
@@ -120,13 +118,13 @@ static bool parseInlineSuppressionCommentToken(const simplecpp::Token *tok, std:
             comment.substr(pos1 + cppchecksuppress.size() + 1, argumentLength);
 
         if ("file" == suppressTypeString)
-            errorType = Suppressions::Type::file;
+            errorType = SuppressionList::Type::file;
         else if ("begin" == suppressTypeString)
-            errorType = Suppressions::Type::blockBegin;
+            errorType = SuppressionList::Type::blockBegin;
         else if ("end" == suppressTypeString)
-            errorType = Suppressions::Type::blockEnd;
+            errorType = SuppressionList::Type::blockEnd;
         else if ("macro" == suppressTypeString)
-            errorType = Suppressions::Type::macro;
+            errorType = SuppressionList::Type::macro;
         else
             return false;
     }
@@ -134,9 +132,9 @@ static bool parseInlineSuppressionCommentToken(const simplecpp::Token *tok, std:
     if (comment[pos2] == '[') {
         // multi suppress format
         std::string errmsg;
-        std::vector<Suppressions::Suppression> suppressions = Suppressions::parseMultiSuppressComment(comment, &errmsg);
+        std::vector<SuppressionList::Suppression> suppressions = SuppressionList::parseMultiSuppressComment(comment, &errmsg);
 
-        for (Suppressions::Suppression &s : suppressions) {
+        for (SuppressionList::Suppression &s : suppressions) {
             s.type = errorType;
             s.lineNumber = tok->location.line;
         }
@@ -144,13 +142,13 @@ static bool parseInlineSuppressionCommentToken(const simplecpp::Token *tok, std:
         if (!errmsg.empty())
             bad.emplace_back(tok->location.file(), tok->location.line, std::move(errmsg));
 
-        std::copy_if(suppressions.cbegin(), suppressions.cend(), std::back_inserter(inlineSuppressions), [](const Suppressions::Suppression& s) {
+        std::copy_if(suppressions.cbegin(), suppressions.cend(), std::back_inserter(inlineSuppressions), [](const SuppressionList::Suppression& s) {
             return !s.errorId.empty();
         });
     } else {
         //single suppress format
         std::string errmsg;
-        Suppressions::Suppression s;
+        SuppressionList::Suppression s;
         if (!s.parseComment(comment, &errmsg))
             return false;
 
@@ -167,9 +165,22 @@ static bool parseInlineSuppressionCommentToken(const simplecpp::Token *tok, std:
     return true;
 }
 
-static void addInlineSuppressions(const simplecpp::TokenList &tokens, const Settings &settings, Suppressions &suppressions, std::list<BadInlineSuppression> &bad)
+static std::string getRelativeFilename(const simplecpp::Token* tok, const Settings &settings) {
+    std::string relativeFilename(tok->location.file());
+    if (settings.relativePaths) {
+        for (const std::string & basePath : settings.basePaths) {
+            const std::string bp = basePath + "/";
+            if (relativeFilename.compare(0,bp.size(),bp)==0) {
+                relativeFilename = relativeFilename.substr(bp.size());
+            }
+        }
+    }
+    return Path::simplifyPath(std::move(relativeFilename));
+}
+
+static void addInlineSuppressions(const simplecpp::TokenList &tokens, const Settings &settings, SuppressionList &suppressions, std::list<BadInlineSuppression> &bad)
 {
-    std::list<Suppressions::Suppression> inlineSuppressionsBlockBegin;
+    std::list<SuppressionList::Suppression> inlineSuppressionsBlockBegin;
 
     bool onlyComments = true;
 
@@ -179,7 +190,7 @@ static void addInlineSuppressions(const simplecpp::TokenList &tokens, const Sett
             continue;
         }
 
-        std::list<Suppressions::Suppression> inlineSuppressions;
+        std::list<SuppressionList::Suppression> inlineSuppressions;
         if (!parseInlineSuppressionCommentToken(tok, inlineSuppressions, bad))
             continue;
 
@@ -207,16 +218,7 @@ static void addInlineSuppressions(const simplecpp::TokenList &tokens, const Sett
             continue;
 
         // Relative filename
-        std::string relativeFilename(tok->location.file());
-        if (settings.relativePaths) {
-            for (const std::string & basePath : settings.basePaths) {
-                const std::string bp = basePath + "/";
-                if (relativeFilename.compare(0,bp.size(),bp)==0) {
-                    relativeFilename = relativeFilename.substr(bp.size());
-                }
-            }
-        }
-        relativeFilename = Path::simplifyPath(relativeFilename);
+        const std::string relativeFilename = getRelativeFilename(tok, settings);
 
         // Macro name
         std::string macroName;
@@ -228,17 +230,17 @@ static void addInlineSuppressions(const simplecpp::TokenList &tokens, const Sett
         }
 
         // Add the suppressions.
-        for (Suppressions::Suppression &suppr : inlineSuppressions) {
+        for (SuppressionList::Suppression &suppr : inlineSuppressions) {
             suppr.fileName = relativeFilename;
 
-            if (Suppressions::Type::blockBegin == suppr.type)
+            if (SuppressionList::Type::blockBegin == suppr.type)
             {
                 inlineSuppressionsBlockBegin.push_back(std::move(suppr));
-            } else if (Suppressions::Type::blockEnd == suppr.type) {
+            } else if (SuppressionList::Type::blockEnd == suppr.type) {
                 bool throwError = true;
 
                 if (!inlineSuppressionsBlockBegin.empty()) {
-                    const Suppressions::Suppression lastBeginSuppression = inlineSuppressionsBlockBegin.back();
+                    const SuppressionList::Suppression lastBeginSuppression = inlineSuppressionsBlockBegin.back();
 
                     auto supprBegin = inlineSuppressionsBlockBegin.begin();
                     while (supprBegin != inlineSuppressionsBlockBegin.end())
@@ -252,7 +254,7 @@ static void addInlineSuppressions(const simplecpp::TokenList &tokens, const Sett
                             suppr.lineBegin = supprBegin->lineNumber;
                             suppr.lineEnd = suppr.lineNumber;
                             suppr.lineNumber = supprBegin->lineNumber;
-                            suppr.type = Suppressions::Type::block;
+                            suppr.type = SuppressionList::Type::block;
                             inlineSuppressionsBlockBegin.erase(supprBegin);
                             suppressions.addSuppression(std::move(suppr));
                             throwError = false;
@@ -266,7 +268,7 @@ static void addInlineSuppressions(const simplecpp::TokenList &tokens, const Sett
                     // NOLINTNEXTLINE(bugprone-use-after-move) - moved only when thrownError is false
                     bad.emplace_back(suppr.fileName, suppr.lineNumber, "Suppress End: No matching begin");
                 }
-            } else if (Suppressions::Type::unique == suppr.type || suppr.type == Suppressions::Type::macro) {
+            } else if (SuppressionList::Type::unique == suppr.type || suppr.type == SuppressionList::Type::macro) {
                 // special handling when suppressing { warnings for backwards compatibility
                 const bool thisAndNextLine = tok->previous &&
                                              tok->previous->previous &&
@@ -280,7 +282,7 @@ static void addInlineSuppressions(const simplecpp::TokenList &tokens, const Sett
                 suppr.lineNumber = tok->location.line;
                 suppr.macroName = macroName;
                 suppressions.addSuppression(std::move(suppr));
-            } else if (Suppressions::Type::file == suppr.type) {
+            } else if (SuppressionList::Type::file == suppr.type) {
                 if (onlyComments)
                     suppressions.addSuppression(std::move(suppr));
                 else
@@ -289,12 +291,12 @@ static void addInlineSuppressions(const simplecpp::TokenList &tokens, const Sett
         }
     }
 
-    for (const Suppressions::Suppression & suppr: inlineSuppressionsBlockBegin)
+    for (const SuppressionList::Suppression & suppr: inlineSuppressionsBlockBegin)
         // cppcheck-suppress useStlAlgorithm
         bad.emplace_back(suppr.fileName, suppr.lineNumber, "Suppress Begin: No matching end");
 }
 
-void Preprocessor::inlineSuppressions(const simplecpp::TokenList &tokens, Suppressions &suppressions)
+void Preprocessor::inlineSuppressions(const simplecpp::TokenList &tokens, SuppressionList &suppressions)
 {
     if (!mSettings.inlineSuppressions)
         return;
@@ -309,10 +311,21 @@ void Preprocessor::inlineSuppressions(const simplecpp::TokenList &tokens, Suppre
     }
 }
 
-void Preprocessor::setDirectives(const simplecpp::TokenList &tokens)
+std::vector<RemarkComment> Preprocessor::getRemarkComments(const simplecpp::TokenList &tokens) const
+{
+    std::vector<RemarkComment> ret;
+    addRemarkComments(tokens, ret);
+    for (std::map<std::string,simplecpp::TokenList*>::const_iterator it = mTokenLists.cbegin(); it != mTokenLists.cend(); ++it) {
+        if (it->second)
+            addRemarkComments(*it->second, ret);
+    }
+    return ret;
+}
+
+std::list<Directive> Preprocessor::createDirectives(const simplecpp::TokenList &tokens) const
 {
     // directive list..
-    mDirectives.clear();
+    std::list<Directive> directives;
 
     std::vector<const simplecpp::TokenList *> list;
     list.reserve(1U + mTokenLists.size());
@@ -327,7 +340,7 @@ void Preprocessor::setDirectives(const simplecpp::TokenList &tokens)
                 continue;
             if (tok->next && tok->next->str() == "endfile")
                 continue;
-            Directive directive(tok->location.file(), tok->location.line, emptyString);
+            Directive directive(tok->location, emptyString);
             for (const simplecpp::Token *tok2 = tok; tok2 && tok2->location.line == directive.linenr; tok2 = tok2->next) {
                 if (tok2->comment)
                     continue;
@@ -337,10 +350,14 @@ void Preprocessor::setDirectives(const simplecpp::TokenList &tokens)
                     directive.str += "include";
                 else
                     directive.str += tok2->str();
+
+                directive.strTokens.emplace_back(*tok2);
             }
-            mDirectives.push_back(std::move(directive));
+            directives.push_back(std::move(directive));
         }
     }
+
+    return directives;
 }
 
 static std::string readcondition(const simplecpp::Token *iftok, const std::set<std::string> &defined, const std::set<std::string> &undefined)
@@ -552,7 +569,7 @@ static void getConfigs(const simplecpp::TokenList &tokens, std::set<std::string>
             }
 
             configs_if.push_back((cmdtok->str() == "ifndef") ? std::string() : config);
-            configs_ifndef.push_back((cmdtok->str() == "ifndef") ? config : std::string());
+            configs_ifndef.push_back((cmdtok->str() == "ifndef") ? std::move(config) : std::string());
             ret.insert(cfg(configs_if,userDefines));
         } else if (cmdtok->str() == "elif" || cmdtok->str() == "else") {
             if (getConfigsElseIsFalse(configs_if,userDefines)) {
@@ -653,41 +670,6 @@ std::set<std::string> Preprocessor::getConfigs(const simplecpp::TokenList &token
     return ret;
 }
 
-
-void Preprocessor::preprocess(std::istream &istr, std::map<std::string, std::string> &result, const std::string &filename, const std::list<std::string> &includePaths)
-{
-    (void)includePaths;
-
-    simplecpp::OutputList outputList;
-    std::vector<std::string> files;
-    const simplecpp::TokenList tokens1(istr, files, filename, &outputList);
-
-    const std::set<std::string> configs = getConfigs(tokens1);
-
-    for (const std::string &c : configs) {
-        if (mSettings.userUndefs.find(c) == mSettings.userUndefs.end()) {
-            result[c] = getcode(tokens1, c, files, false);
-        }
-    }
-}
-
-void Preprocessor::preprocess(std::istream &srcCodeStream, std::string &processedFile, std::list<std::string> &resultConfigurations, const std::string &filename, const std::list<std::string> &includePaths)
-{
-    (void)includePaths;
-
-    if (mFile0.empty())
-        mFile0 = filename;
-
-    simplecpp::OutputList outputList;
-    std::vector<std::string> files;
-    const simplecpp::TokenList tokens1(srcCodeStream, files, filename, &outputList);
-
-    const std::set<std::string> configs = getConfigs(tokens1);
-    std::copy(configs.cbegin(), configs.cend(), std::back_inserter(resultConfigurations));
-
-    processedFile = tokens1.stringify();
-}
-
 static void splitcfg(const std::string &cfg, std::list<std::string> &defines, const std::string &defaultValue)
 {
     for (std::string::size_type defineStartPos = 0U; defineStartPos < cfg.size();) {
@@ -711,7 +693,7 @@ static simplecpp::DUI createDUI(const Settings &mSettings, const std::string &cf
     if (!cfg.empty())
         splitcfg(cfg, dui.defines, emptyString);
 
-    for (const std::string &def : mSettings.library.defines) {
+    for (const std::string &def : mSettings.library.defines()) {
         const std::string::size_type pos = def.find_first_of(" (");
         if (pos == std::string::npos) {
             dui.defines.push_back(def);
@@ -730,11 +712,13 @@ static simplecpp::DUI createDUI(const Settings &mSettings, const std::string &cf
     dui.includePaths = mSettings.includePaths; // -I
     dui.includes = mSettings.userIncludes;  // --include
     // TODO: use mSettings.standards.stdValue instead
-    if (Path::isCPP(filename)) {
+    // TODO: error out on unknown language?
+    const Standards::Language lang = Path::identify(filename, mSettings.cppHeaderProbe);
+    if (lang == Standards::Language::CPP) {
         dui.std = mSettings.standards.getCPP();
         splitcfg(mSettings.platform.getLimitsDefines(Standards::getCPP(dui.std)), dui.defines, "");
     }
-    else {
+    else if (lang == Standards::Language::C) {
         dui.std = mSettings.standards.getC();
         splitcfg(mSettings.platform.getLimitsDefines(Standards::getC(dui.std)), dui.defines, "");
     }
@@ -751,6 +735,7 @@ bool Preprocessor::hasErrors(const simplecpp::Output &output)
     case simplecpp::Output::UNHANDLED_CHAR_ERROR:
     case simplecpp::Output::EXPLICIT_INCLUDE_NOT_FOUND:
     case simplecpp::Output::FILE_NOT_FOUND:
+    case simplecpp::Output::DUI_ERROR:
         return true;
     case simplecpp::Output::WARNING:
     case simplecpp::Output::MISSING_HEADER:
@@ -792,32 +777,33 @@ bool Preprocessor::loadFiles(const simplecpp::TokenList &rawtokens, std::vector<
     return !hasErrors(outputList);
 }
 
-void Preprocessor::removeComments()
+void Preprocessor::removeComments(simplecpp::TokenList &tokens)
 {
+    tokens.removeComments();
     for (std::pair<const std::string, simplecpp::TokenList*>& tokenList : mTokenLists) {
         if (tokenList.second)
             tokenList.second->removeComments();
     }
 }
 
-void Preprocessor::setPlatformInfo(simplecpp::TokenList *tokens) const
+void Preprocessor::setPlatformInfo(simplecpp::TokenList &tokens, const Settings& settings)
 {
-    tokens->sizeOfType["bool"]          = mSettings.platform.sizeof_bool;
-    tokens->sizeOfType["short"]         = mSettings.platform.sizeof_short;
-    tokens->sizeOfType["int"]           = mSettings.platform.sizeof_int;
-    tokens->sizeOfType["long"]          = mSettings.platform.sizeof_long;
-    tokens->sizeOfType["long long"]     = mSettings.platform.sizeof_long_long;
-    tokens->sizeOfType["float"]         = mSettings.platform.sizeof_float;
-    tokens->sizeOfType["double"]        = mSettings.platform.sizeof_double;
-    tokens->sizeOfType["long double"]   = mSettings.platform.sizeof_long_double;
-    tokens->sizeOfType["bool *"]        = mSettings.platform.sizeof_pointer;
-    tokens->sizeOfType["short *"]       = mSettings.platform.sizeof_pointer;
-    tokens->sizeOfType["int *"]         = mSettings.platform.sizeof_pointer;
-    tokens->sizeOfType["long *"]        = mSettings.platform.sizeof_pointer;
-    tokens->sizeOfType["long long *"]   = mSettings.platform.sizeof_pointer;
-    tokens->sizeOfType["float *"]       = mSettings.platform.sizeof_pointer;
-    tokens->sizeOfType["double *"]      = mSettings.platform.sizeof_pointer;
-    tokens->sizeOfType["long double *"] = mSettings.platform.sizeof_pointer;
+    tokens.sizeOfType["bool"]          = settings.platform.sizeof_bool;
+    tokens.sizeOfType["short"]         = settings.platform.sizeof_short;
+    tokens.sizeOfType["int"]           = settings.platform.sizeof_int;
+    tokens.sizeOfType["long"]          = settings.platform.sizeof_long;
+    tokens.sizeOfType["long long"]     = settings.platform.sizeof_long_long;
+    tokens.sizeOfType["float"]         = settings.platform.sizeof_float;
+    tokens.sizeOfType["double"]        = settings.platform.sizeof_double;
+    tokens.sizeOfType["long double"]   = settings.platform.sizeof_long_double;
+    tokens.sizeOfType["bool *"]        = settings.platform.sizeof_pointer;
+    tokens.sizeOfType["short *"]       = settings.platform.sizeof_pointer;
+    tokens.sizeOfType["int *"]         = settings.platform.sizeof_pointer;
+    tokens.sizeOfType["long *"]        = settings.platform.sizeof_pointer;
+    tokens.sizeOfType["long long *"]   = settings.platform.sizeof_pointer;
+    tokens.sizeOfType["float *"]       = settings.platform.sizeof_pointer;
+    tokens.sizeOfType["double *"]      = settings.platform.sizeof_pointer;
+    tokens.sizeOfType["long double *"] = settings.platform.sizeof_pointer;
 }
 
 simplecpp::TokenList Preprocessor::preprocess(const simplecpp::TokenList &tokens1, const std::string &cfg, std::vector<std::string> &files, bool throwError)
@@ -829,8 +815,8 @@ simplecpp::TokenList Preprocessor::preprocess(const simplecpp::TokenList &tokens
     std::list<simplecpp::IfCond> ifCond;
     simplecpp::TokenList tokens2(files);
     simplecpp::preprocess(tokens2, tokens1, files, mTokenLists, dui, &outputList, &macroUsage, &ifCond);
-    mMacroUsage = macroUsage;
-    mIfCond = ifCond;
+    mMacroUsage = std::move(macroUsage);
+    mIfCond = std::move(ifCond);
 
     handleErrors(outputList, throwError);
 
@@ -891,6 +877,7 @@ void Preprocessor::reportOutput(const simplecpp::OutputList &outputList, bool sh
             break;
         case simplecpp::Output::EXPLICIT_INCLUDE_NOT_FOUND:
         case simplecpp::Output::FILE_NOT_FOUND:
+        case simplecpp::Output::DUI_ERROR:
             error(emptyString, 0, out.msg);
             break;
         }
@@ -905,26 +892,25 @@ void Preprocessor::error(const std::string &filename, unsigned int linenr, const
         if (mSettings.relativePaths)
             file = Path::getRelativePath(file, mSettings.basePaths);
 
-        ErrorMessage::FileLocation loc(file, linenr, 0);
-        locationList.push_back(std::move(loc));
+        locationList.emplace_back(file, linenr, 0);
     }
-    mErrorLogger->reportErr(ErrorMessage(locationList,
-                                         mFile0,
-                                         Severity::error,
-                                         msg,
-                                         "preprocessorErrorDirective",
-                                         Certainty::normal));
+    mErrorLogger.reportErr(ErrorMessage(std::move(locationList),
+                                        mFile0,
+                                        Severity::error,
+                                        msg,
+                                        "preprocessorErrorDirective",
+                                        Certainty::normal));
 }
 
 // Report that include is missing
 void Preprocessor::missingInclude(const std::string &filename, unsigned int linenr, const std::string &header, HeaderTypes headerType)
 {
-    if (!mSettings.checks.isEnabled(Checks::missingInclude) || !mErrorLogger)
+    if (!mSettings.checks.isEnabled(Checks::missingInclude))
         return;
 
     std::list<ErrorMessage::FileLocation> locationList;
     if (!filename.empty()) {
-        locationList.emplace_back(filename, linenr);
+        locationList.emplace_back(filename, linenr, 0);
     }
     ErrorMessage errmsg(std::move(locationList), mFile0, Severity::information,
                         (headerType==SystemHeader) ?
@@ -932,13 +918,12 @@ void Preprocessor::missingInclude(const std::string &filename, unsigned int line
                         "Include file: \"" + header + "\" not found.",
                         (headerType==SystemHeader) ? "missingIncludeSystem" : "missingInclude",
                         Certainty::normal);
-    mErrorLogger->reportErr(errmsg);
+    mErrorLogger.reportErr(errmsg);
 }
 
-void Preprocessor::getErrorMessages(ErrorLogger *errorLogger, const Settings *settings)
+void Preprocessor::getErrorMessages(ErrorLogger &errorLogger, const Settings &settings)
 {
-    Settings settings2(*settings);
-    Preprocessor preprocessor(settings2, errorLogger);
+    Preprocessor preprocessor(settings, errorLogger);
     preprocessor.missingInclude(emptyString, 1, emptyString, UserHeader);
     preprocessor.missingInclude(emptyString, 1, emptyString, SystemHeader);
     preprocessor.error(emptyString, 1, "#error message");   // #error ..
@@ -948,17 +933,6 @@ void Preprocessor::dump(std::ostream &out) const
 {
     // Create a xml dump.
 
-    out << "  <directivelist>" << std::endl;
-    for (const Directive &dir : mDirectives) {
-        out << "    <directive "
-            << "file=\"" << ErrorLogger::toxml(dir.file) << "\" "
-            << "linenr=\"" << dir.linenr << "\" "
-            // str might contain characters such as '"', '<' or '>' which
-            // could result in invalid XML, so run it through toxml().
-            << "str=\"" << ErrorLogger::toxml(dir.str) << "\"/>" << std::endl;
-    }
-    out << "  </directivelist>" << std::endl;
-
     if (!mMacroUsage.empty()) {
         out << "  <macro-usage>" << std::endl;
         for (const simplecpp::MacroUsage &macroUsage: mMacroUsage) {
@@ -995,30 +969,36 @@ std::size_t Preprocessor::calculateHash(const simplecpp::TokenList &tokens1, con
 {
     std::string hashData = toolinfo;
     for (const simplecpp::Token *tok = tokens1.cfront(); tok; tok = tok->next) {
-        if (!tok->comment)
+        if (!tok->comment) {
             hashData += tok->str();
+            hashData += static_cast<char>(tok->location.line);
+            hashData += static_cast<char>(tok->location.col);
+        }
     }
     for (std::map<std::string, simplecpp::TokenList *>::const_iterator it = mTokenLists.cbegin(); it != mTokenLists.cend(); ++it) {
         for (const simplecpp::Token *tok = it->second->cfront(); tok; tok = tok->next) {
-            if (!tok->comment)
+            if (!tok->comment) {
                 hashData += tok->str();
+                hashData += static_cast<char>(tok->location.line);
+                hashData += static_cast<char>(tok->location.col);
+            }
         }
     }
     return (std::hash<std::string>{})(hashData);
 }
 
-void Preprocessor::simplifyPragmaAsm(simplecpp::TokenList *tokenList) const
+void Preprocessor::simplifyPragmaAsm(simplecpp::TokenList &tokenList) const
 {
     Preprocessor::simplifyPragmaAsmPrivate(tokenList);
     for (const std::pair<const std::string, simplecpp::TokenList*>& list : mTokenLists) {
-        Preprocessor::simplifyPragmaAsmPrivate(list.second);
+        Preprocessor::simplifyPragmaAsmPrivate(*list.second);
     }
 }
 
-void Preprocessor::simplifyPragmaAsmPrivate(simplecpp::TokenList *tokenList)
+void Preprocessor::simplifyPragmaAsmPrivate(simplecpp::TokenList &tokenList)
 {
     // assembler code..
-    for (simplecpp::Token *tok = tokenList->front(); tok; tok = tok->next) {
+    for (simplecpp::Token *tok = tokenList.front(); tok; tok = tok->next) {
         if (tok->op != '#')
             continue;
         if (sameline(tok, tok->previousSkipComments()))
@@ -1053,6 +1033,55 @@ void Preprocessor::simplifyPragmaAsmPrivate(simplecpp::TokenList *tokenList)
         const_cast<simplecpp::Token *>(tok3)->setstr(")");
         const_cast<simplecpp::Token *>(tok4)->setstr(";");
         while (tok4->next != endasm)
-            tokenList->deleteToken(tok4->next);
+            tokenList.deleteToken(tok4->next);
+    }
+}
+
+
+void Preprocessor::addRemarkComments(const simplecpp::TokenList &tokens, std::vector<RemarkComment> &remarkComments) const
+{
+    for (const simplecpp::Token *tok = tokens.cfront(); tok; tok = tok->next) {
+        if (!tok->comment)
+            continue;
+
+        const std::string& comment = tok->str();
+
+        // is it a remark comment?
+        const std::string::size_type pos1 = comment.find_first_not_of("/* \t");
+        if (pos1 == std::string::npos)
+            continue;
+        const std::string::size_type pos2 = comment.find_first_of(": \t", pos1);
+        if (pos2 != pos1 + 6 || comment.compare(pos1, 6, "REMARK") != 0)
+            continue;
+        const std::string::size_type pos3 = comment.find_first_not_of(": \t", pos2);
+        if (pos3 == std::string::npos)
+            continue;
+        if (comment.compare(0,2,"/*") == 0 && pos3 + 2 >= tok->str().size())
+            continue;
+
+        const std::string::size_type pos4 = (comment.compare(0,2,"/*") == 0) ? comment.size()-2 : comment.size();
+        const std::string remarkText = comment.substr(pos3, pos4-pos3);
+
+        // Get remarked token
+        const simplecpp::Token* remarkedToken = nullptr;
+        for (const simplecpp::Token* after = tok->next; after; after = after->next) {
+            if (after->comment)
+                continue;
+            remarkedToken = after;
+            break;
+        }
+        for (const simplecpp::Token* prev = tok->previous; prev; prev = prev->previous) {
+            if (prev->comment)
+                continue;
+            if (sameline(prev, tok))
+                remarkedToken = prev;
+            break;
+        }
+
+        // Relative filename
+        const std::string relativeFilename = getRelativeFilename(remarkedToken, mSettings);
+
+        // Add the suppressions.
+        remarkComments.emplace_back(relativeFilename, remarkedToken->location.line, remarkText);
     }
 }
