@@ -1,4 +1,4 @@
-/*
+/* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
  * Copyright (C) 2007-2024 Cppcheck team.
  *
@@ -22,11 +22,13 @@
 //---------------------------------------------------------------------------
 
 #include "config.h"
+#include "errortypes.h"
 #include "mathlib.h"
 #include "templatesimplifier.h"
 #include "utils.h"
 #include "vfvalue.h"
 
+#include <algorithm>
 #include <cassert>
 #include <cstdint>
 #include <cstddef>
@@ -36,6 +38,7 @@
 #include <ostream>
 #include <set>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -57,7 +60,7 @@ struct ScopeInfo2 {
     std::set<std::string> usingNamespaces;
 };
 
-enum class TokenDebug { None, ValueFlow, ValueType };
+enum class TokenDebug : std::uint8_t { None, ValueFlow, ValueType };
 
 struct TokenImpl {
     nonneg int mVarId{};
@@ -115,7 +118,7 @@ struct TokenImpl {
 
     // __cppcheck_in_range__
     struct CppcheckAttributes {
-        enum Type { LOW, HIGH } type = LOW;
+        enum Type : std::uint8_t { LOW, HIGH } type = LOW;
         MathLib::bigint value{};
         CppcheckAttributes* next{};
     };
@@ -130,8 +133,10 @@ struct TokenImpl {
             mAttributeAlignas->push_back(a);
     }
 
+    std::string mAttributeCleanup;
+
     // For memoization, to speed up parsing of huge arrays #8897
-    enum class Cpp11init { UNKNOWN, CPP11INIT, NOINIT } mCpp11init = Cpp11init::UNKNOWN;
+    enum class Cpp11init : std::uint8_t { UNKNOWN, CPP11INIT, NOINIT } mCpp11init = Cpp11init::UNKNOWN;
 
     TokenDebug mDebug{};
 
@@ -166,7 +171,7 @@ public:
     Token(const Token &) = delete;
     Token& operator=(const Token &) = delete;
 
-    enum Type {
+    enum Type : std::uint8_t {
         eVariable, eType, eFunction, eKeyword, eName, // Names: Variable (varId), Type (typeId, later), Function (FuncId, later), Language keyword, Name (unknown identifier)
         eNumber, eString, eChar, eBoolean, eLiteral, eEnumerator, // Literals: Number, String, Character, Boolean, User defined literal (C++11), Enumerator
         eArithmeticalOp, eComparisonOp, eAssignmentOp, eLogicalOp, eBitOp, eIncDecOp, eExtendedOp, // Operators: Arithmetical, Comparison, Assignment, Logical, Bitwise, ++/--, Extended
@@ -222,21 +227,37 @@ public:
      * For example index 1 would return next token, and 2
      * would return next from that one.
      */
-    const Token *tokAt(int index) const;
-    Token *tokAt(int index);
+    const Token *tokAt(int index) const
+    {
+        return tokAtImpl(this, index);
+    }
+    Token *tokAt(int index)
+    {
+        return tokAtImpl(this, index);
+    }
 
     /**
      * @return the link to the token in given index, related to this token.
      * For example index 1 would return the link to next token.
      */
-    const Token *linkAt(int index) const;
-    Token *linkAt(int index);
+    const Token *linkAt(int index) const
+    {
+        return linkAtImpl(this, index);
+    }
+    Token *linkAt(int index)
+    {
+        return linkAtImpl(this, index);
+    }
 
     /**
      * @return String of the token in given index, related to this token.
      * If that token does not exist, an empty string is being returned.
      */
-    const std::string &strAt(int index) const;
+    const std::string &strAt(int index) const
+    {
+        const Token *tok = this->tokAt(index);
+        return tok ? tok->mStr : emptyString;
+    }
 
     /**
      * Match given token (or list of tokens) to a pattern list.
@@ -342,12 +363,7 @@ public:
     }
     void setValueType(ValueType *vt);
 
-    const ValueType *argumentType() const {
-        const Token *top = this;
-        while (top && !Token::Match(top->astParent(), ",|("))
-            top = top->astParent();
-        return top ? top->mImpl->mValueType : nullptr;
-    }
+    const ValueType *argumentType() const;
 
     Token::Type tokType() const {
         return mTokType;
@@ -551,6 +567,15 @@ public:
     void addAttributeAlignas(const std::string& a) {
         mImpl->addAttributeAlignas(a);
     }
+    void addAttributeCleanup(const std::string& funcname) {
+        mImpl->mAttributeCleanup = funcname;
+    }
+    const std::string& getAttributeCleanup() const {
+        return mImpl->mAttributeCleanup;
+    }
+    bool hasAttributeCleanup() const {
+        return !mImpl->mAttributeCleanup.empty();
+    }
     void setCppcheckAttribute(TokenImpl::CppcheckAttributes::Type type, MathLib::bigint value) {
         mImpl->setCppcheckAttribute(type, value);
     }
@@ -739,13 +764,13 @@ public:
 
     bool isCChar() const {
         return (((mTokType == eString) && isPrefixStringCharLiteral(mStr, '"', emptyString)) ||
-                ((mTokType ==  eChar) && isPrefixStringCharLiteral(mStr, '\'', emptyString) && mStr.length() == 3));
+                ((mTokType == eChar) && isPrefixStringCharLiteral(mStr, '\'', emptyString) && (replaceEscapeSequences(getCharLiteral(mStr)).size() == 1)));
     }
 
     bool isCMultiChar() const {
-        return (((mTokType ==  eChar) && isPrefixStringCharLiteral(mStr, '\'', emptyString)) &&
-                (mStr.length() > 3));
+        return (mTokType == eChar) && isPrefixStringCharLiteral(mStr, '\'', emptyString) && (replaceEscapeSequences(getCharLiteral(mStr)).size() > 1);
     }
+
     /**
      * @brief Is current token a template argument?
      *
@@ -770,7 +795,7 @@ public:
         setFlag(fIsTemplateArg, value);
     }
 
-    std::string getMacroName() const {
+    const std::string& getMacroName() const {
         return mImpl->mMacroName;
     }
     void setMacroName(std::string name) {
@@ -781,32 +806,56 @@ public:
     static const Token *findsimplematch(const Token * const startTok, const char (&pattern)[count]) {
         return findsimplematch(startTok, pattern, count-1);
     }
-    static const Token *findsimplematch(const Token * const startTok, const char pattern[], size_t pattern_len);
+    static const Token *findsimplematch(const Token * startTok, const char pattern[], size_t pattern_len);
 
     template<size_t count>
     static const Token *findsimplematch(const Token * const startTok, const char (&pattern)[count], const Token * const end) {
         return findsimplematch(startTok, pattern, count-1, end);
     }
-    static const Token *findsimplematch(const Token * const startTok, const char pattern[], size_t pattern_len, const Token * const end);
+    static const Token *findsimplematch(const Token * startTok, const char pattern[], size_t pattern_len, const Token * end);
 
-    static const Token *findmatch(const Token * const startTok, const char pattern[], const nonneg int varId = 0);
-    static const Token *findmatch(const Token * const startTok, const char pattern[], const Token * const end, const nonneg int varId = 0);
+    static const Token *findmatch(const Token * startTok, const char pattern[], nonneg int varId = 0);
+    static const Token *findmatch(const Token * startTok, const char pattern[], const Token * end, nonneg int varId = 0);
 
     template<size_t count>
     static Token *findsimplematch(Token * const startTok, const char (&pattern)[count]) {
         return findsimplematch(startTok, pattern, count-1);
     }
-    static Token *findsimplematch(Token * const startTok, const char pattern[], size_t pattern_len);
+    static Token *findsimplematch(Token * startTok, const char pattern[], size_t pattern_len);
     template<size_t count>
     static Token *findsimplematch(Token * const startTok, const char (&pattern)[count], const Token * const end) {
         return findsimplematch(startTok, pattern, count-1, end);
     }
-    static Token *findsimplematch(Token * const startTok, const char pattern[], size_t pattern_len, const Token * const end);
+    static Token *findsimplematch(Token * startTok, const char pattern[], size_t pattern_len, const Token * end);
 
-    static Token *findmatch(Token * const startTok, const char pattern[], const nonneg int varId = 0);
-    static Token *findmatch(Token * const startTok, const char pattern[], const Token * const end, const nonneg int varId = 0);
+    static Token *findmatch(Token * startTok, const char pattern[], nonneg int varId = 0);
+    static Token *findmatch(Token * startTok, const char pattern[], const Token * end, nonneg int varId = 0);
 
 private:
+    template<class T, REQUIRES("T must be a Token class", std::is_convertible<T*, const Token*> )>
+    static T *tokAtImpl(T *tok, int index)
+    {
+        while (index > 0 && tok) {
+            tok = tok->next();
+            --index;
+        }
+        while (index < 0 && tok) {
+            tok = tok->previous();
+            ++index;
+        }
+        return tok;
+    }
+
+    template<class T, REQUIRES("T must be a Token class", std::is_convertible<T*, const Token*> )>
+    static T *linkAtImpl(T *thisTok, int index)
+    {
+        T *tok = thisTok->tokAt(index);
+        if (!tok) {
+            throw InternalError(thisTok, "Internal error. Token::linkAt called with index outside the tokens range.");
+        }
+        return tok->link();
+    }
+
     /**
      * Needle is build from multiple alternatives. If one of
      * them is equal to haystack, return value is 1. If there
@@ -824,6 +873,8 @@ private:
     static int multiCompare(const Token *tok, const char *haystack, nonneg int varid);
 
 public:
+    const std::string& fileName() const;
+
     nonneg int fileIndex() const {
         return mImpl->mFileIndex;
     }
@@ -870,9 +921,9 @@ public:
      * @param prepend Insert the new token before this token when it's not
      * the first one on the tokens list.
      */
-    Token* insertToken(const std::string& tokenStr, const std::string& originalNameStr = emptyString, const std::string& macroNameStr = emptyString, bool prepend = false);
+    RET_NONNULL Token* insertToken(const std::string& tokenStr, const std::string& originalNameStr = emptyString, const std::string& macroNameStr = emptyString, bool prepend = false);
 
-    Token* insertTokenBefore(const std::string& tokenStr, const std::string& originalNameStr = emptyString, const std::string& macroNameStr = emptyString)
+    RET_NONNULL Token* insertTokenBefore(const std::string& tokenStr, const std::string& originalNameStr = emptyString, const std::string& macroNameStr = emptyString)
     {
         return insertToken(tokenStr, originalNameStr, macroNameStr, true);
     }
@@ -919,27 +970,30 @@ public:
     }
 
     /**
-     * For debugging purposes, prints token and all tokens
-     * followed by it.
+     * For debugging purposes, prints token and all tokens followed by it.
+     */
+    void printOut() const;
+
+    /**
+     * For debugging purposes, prints token and all tokens followed by it.
      * @param title Title for the printout or use default parameter or 0
      * for no title.
      */
-    void printOut(const char *title = nullptr) const;
+    void printOut(std::ostream& out, const char *title = nullptr) const;
 
     /**
-     * For debugging purposes, prints token and all tokens
-     * followed by it.
+     * For debugging purposes, prints token and all tokens followed by it.
      * @param title Title for the printout or use default parameter or 0
      * for no title.
      * @param fileNames Prints out file name instead of file index.
      * File index should match the index of the string in this vector.
      */
-    void printOut(const char *title, const std::vector<std::string> &fileNames) const;
+    void printOut(std::ostream& out, const char *title, const std::vector<std::string> &fileNames) const;
 
     /**
      * print out tokens - used for debugging
      */
-    void printLines(int lines=5) const;
+    void printLines(std::ostream& out, int lines=5) const;
 
     /**
      * Replace token replaceThis with tokens between start and end,
@@ -1175,7 +1229,7 @@ public:
     /**
      * @return the first token of the next argument. Does only work on argument
      * lists. Requires that Tokenizer::createLinks2() has been called before.
-     * Returns 0, if there is no next argument.
+     * Returns nullptr, if there is no next argument.
      */
     const Token* nextArgument() const;
     Token *nextArgument();
@@ -1183,14 +1237,14 @@ public:
     /**
      * @return the first token of the next argument. Does only work on argument
      * lists. Should be used only before Tokenizer::createLinks2() was called.
-     * Returns 0, if there is no next argument.
+     * Returns nullptr, if there is no next argument.
      */
     const Token* nextArgumentBeforeCreateLinks2() const;
 
     /**
      * @return the first token of the next template argument. Does only work on template argument
      * lists. Requires that Tokenizer::createLinks2() has been called before.
-     * Returns 0, if there is no next argument.
+     * Returns nullptr, if there is no next argument.
      */
     const Token* nextTemplateArgument() const;
 
@@ -1237,19 +1291,19 @@ public:
         return mImpl->mValues->front().intvalue;
     }
 
-    const ValueFlow::Value* getValue(const MathLib::bigint val) const;
+    const ValueFlow::Value* getValue(MathLib::bigint val) const;
 
     const ValueFlow::Value* getMaxValue(bool condition, MathLib::bigint path = 0) const;
     const ValueFlow::Value* getMinValue(bool condition, MathLib::bigint path = 0) const;
 
     const ValueFlow::Value* getMovedValue() const;
 
-    const ValueFlow::Value * getValueLE(const MathLib::bigint val, const Settings *settings) const;
-    const ValueFlow::Value * getValueGE(const MathLib::bigint val, const Settings *settings) const;
+    const ValueFlow::Value * getValueLE(MathLib::bigint val, const Settings &settings) const;
+    const ValueFlow::Value * getValueGE(MathLib::bigint val, const Settings &settings) const;
 
-    const ValueFlow::Value * getInvalidValue(const Token *ftok, nonneg int argnr, const Settings *settings) const;
+    const ValueFlow::Value * getInvalidValue(const Token *ftok, nonneg int argnr, const Settings &settings) const;
 
-    const ValueFlow::Value* getContainerSizeValue(const MathLib::bigint val) const;
+    const ValueFlow::Value* getContainerSizeValue(MathLib::bigint val) const;
 
     const Token *getValueTokenMaxStrLength() const;
     const Token *getValueTokenMinStrSize(const Settings &settings, MathLib::bigint* path = nullptr) const;
@@ -1346,7 +1400,7 @@ private:
         fIsInitComma            = (1ULL << 42), // Is this comma located inside some {..}. i.e: {1,2,3,4}
     };
 
-    enum : uint64_t {
+    enum : std::uint8_t  {
         efMaxSize = sizeof(nonneg int) * 8,
         efIsUnique = efMaxSize - 2,
     };
@@ -1382,11 +1436,8 @@ private:
     /** Update internal property cache about isStandardType() */
     void update_property_isStandardType();
 
-    /** Update internal property cache about string and char literals */
-    void update_property_char_string_literal();
-
     /** Internal helper function to avoid excessive string allocations */
-    void astStringVerboseRecursive(std::string& ret, const nonneg int indent1 = 0, const nonneg int indent2 = 0) const;
+    void astStringVerboseRecursive(std::string& ret, nonneg int indent1 = 0, nonneg int indent2 = 0) const;
 
 public:
     void astOperand1(Token *tok);
@@ -1431,14 +1482,14 @@ public:
         return nullptr;
 
     }
-    Token *astTop() {
+    RET_NONNULL Token *astTop() {
         Token *ret = this;
         while (ret->mImpl->mAstParent)
             ret = ret->mImpl->mAstParent;
         return ret;
     }
 
-    const Token *astTop() const {
+    RET_NONNULL const Token *astTop() const {
         const Token *ret = this;
         while (ret->mImpl->mAstParent)
             ret = ret->mImpl->mAstParent;
