@@ -4,6 +4,8 @@
 import os
 import re
 import glob
+import json
+import xml.etree.ElementTree as ET
 
 from testutils import create_gui_project_file, cppcheck
 
@@ -13,7 +15,7 @@ __proj_dir = os.path.join(__script_dir, 'helloworld')
 
 # Get Visual Studio configurations checking a file
 # Checking {file} {config}...
-def getVsConfigs(stdout, filename):
+def __getVsConfigs(stdout, filename):
     ret = []
     for line in stdout.split('\n'):
         if not line.startswith('Checking %s ' % filename):
@@ -148,7 +150,7 @@ def test_vs_project_local_path():
     ]
     ret, stdout, stderr = cppcheck(args, cwd=__proj_dir)
     assert ret == 0, stdout
-    assert getVsConfigs(stdout, 'main.c') == 'Debug|Win32 Debug|x64 Release|Win32 Release|x64'
+    assert __getVsConfigs(stdout, 'main.c') == 'Debug|Win32 Debug|x64 Release|Win32 Release|x64'
     assert stderr == '[main.c:5]: (error) Division by zero.\n'
 
 def test_vs_project_relative_path():
@@ -159,7 +161,7 @@ def test_vs_project_relative_path():
     ret, stdout, stderr = cppcheck(args, cwd=__script_dir)
     filename = os.path.join(__proj_dir, 'main.c')
     assert ret == 0, stdout
-    assert getVsConfigs(stdout, filename) == 'Debug|Win32 Debug|x64 Release|Win32 Release|x64'
+    assert __getVsConfigs(stdout, filename) == 'Debug|Win32 Debug|x64 Release|Win32 Release|x64'
     assert stderr == '[%s:5]: (error) Division by zero.\n' % filename
 
 def test_vs_project_absolute_path():
@@ -170,7 +172,7 @@ def test_vs_project_absolute_path():
     ret, stdout, stderr = cppcheck(args)
     filename = os.path.join(__proj_dir, 'main.c')
     assert ret == 0, stdout
-    assert getVsConfigs(stdout, filename) == 'Debug|Win32 Debug|x64 Release|Win32 Release|x64'
+    assert __getVsConfigs(stdout, filename) == 'Debug|Win32 Debug|x64 Release|Win32 Release|x64'
     assert stderr == '[%s:5]: (error) Division by zero.\n' % filename
 
 def test_cppcheck_project_local_path():
@@ -181,7 +183,7 @@ def test_cppcheck_project_local_path():
     ]
     ret, stdout, stderr = cppcheck(args, cwd=__proj_dir)
     assert ret == 0, stdout
-    assert getVsConfigs(stdout, 'main.c') == 'Debug|x64'
+    assert __getVsConfigs(stdout, 'main.c') == 'Debug|x64'
     assert stderr == '[main.c:5]: (error) Division by zero.\n'
 
 def test_cppcheck_project_relative_path():
@@ -193,7 +195,7 @@ def test_cppcheck_project_relative_path():
     ret, stdout, stderr = cppcheck(args, cwd=__script_dir)
     filename = os.path.join('helloworld', 'main.c')
     assert ret == 0, stdout
-    assert getVsConfigs(stdout, filename) == 'Debug|x64'
+    assert __getVsConfigs(stdout, filename) == 'Debug|x64'
     assert stderr == '[%s:5]: (error) Division by zero.\n' % filename
 
 def test_cppcheck_project_absolute_path():
@@ -205,7 +207,7 @@ def test_cppcheck_project_absolute_path():
     ret, stdout, stderr = cppcheck(args)
     filename = os.path.join(__proj_dir, 'main.c')
     assert ret == 0, stdout
-    assert getVsConfigs(stdout, filename) == 'Debug|x64'
+    assert __getVsConfigs(stdout, filename) == 'Debug|x64'
     assert stderr == '[%s:5]: (error) Division by zero.\n' % filename
 
 def test_suppress_command_line_related():
@@ -319,3 +321,30 @@ def test_missing_include_system():  # #11283
 
     _, _, stderr = cppcheck(args, cwd=__script_dir)
     assert stderr.replace('\\', '/') == 'helloworld/main.c:1:0: information: Include file: <stdio.h> not found. Please note: Cppcheck does not need standard library headers to get proper results. [missingIncludeSystem]\n'
+
+
+def test_sarif():
+    args = [
+        '--output-format=sarif',
+        'helloworld'
+    ]
+    ret, stdout, stderr = cppcheck(args, cwd=__script_dir)
+    assert ret == 0, stdout
+    res = json.loads(stderr)
+    assert res['version'] == '2.1.0'
+    assert res['runs'][0]['results'][0]['locations'][0]['physicalLocation']['artifactLocation']['uri'] == 'helloworld/main.c'
+    assert res['runs'][0]['results'][0]['ruleId'] == 'zerodiv'
+    assert res['runs'][0]['tool']['driver']['rules'][0]['id'] == 'zerodiv'
+    assert res['runs'][0]['tool']['driver']['rules'][0]['properties']['precision'] == 'high'
+    assert res['runs'][0]['tool']['driver']['rules'][0]['properties']['security-severity'] > 9.5
+    assert 'security' in res['runs'][0]['tool']['driver']['rules'][0]['properties']['tags']
+    assert re.match(r'[0-9]+(.[0-9]+)+', res['runs'][0]['tool']['driver']['semanticVersion'])
+
+
+def test_xml_checkers_report():
+    test_file = os.path.join(__proj_dir, 'main.c')
+    args = ['--xml-version=3', '--enable=all', test_file]
+
+    exitcode, _, stderr = cppcheck(args)
+    assert exitcode == 0
+    assert ET.fromstring(stderr) is not None
