@@ -86,10 +86,10 @@ def test_preprocessor_error(tmpdir):
     assert exitcode != 0
 
 
-ANSI_BOLD = "\x1b[1m"
-ANSI_FG_RED = "\x1b[31m"
-ANSI_FG_DEFAULT = "\x1b[39m"
-ANSI_FG_RESET = "\x1b[0m"
+__ANSI_BOLD = "\x1b[1m"
+__ANSI_FG_RED = "\x1b[31m"
+__ANSI_FG_DEFAULT = "\x1b[39m"
+__ANSI_FG_RESET = "\x1b[0m"
 
 
 @pytest.mark.parametrize("env,color_expected", [({"CLICOLOR_FORCE":"1"}, True), ({"NO_COLOR": "1", "CLICOLOR_FORCE":"1"}, False)])
@@ -101,10 +101,10 @@ def test_color_non_tty(tmpdir, env, color_expected):
 
     assert exitcode == 0
     assert stderr
-    assert (ANSI_BOLD in stderr) == color_expected
-    assert (ANSI_FG_RED in stderr)  == color_expected
-    assert (ANSI_FG_DEFAULT in stderr) == color_expected
-    assert (ANSI_FG_RESET in stderr) == color_expected
+    assert (__ANSI_BOLD in stderr) == color_expected
+    assert (__ANSI_FG_RED in stderr) == color_expected
+    assert (__ANSI_FG_DEFAULT in stderr) == color_expected
+    assert (__ANSI_FG_RESET in stderr) == color_expected
 
 
 @pytest.mark.skipif(sys.platform == "win32", reason="TTY not supported in Windows")
@@ -117,10 +117,10 @@ def test_color_tty(tmpdir, env, color_expected):
 
     assert exitcode == 0
     assert stderr
-    assert (ANSI_BOLD in stderr) == color_expected
-    assert (ANSI_FG_RED in stderr) == color_expected
-    assert (ANSI_FG_DEFAULT in stderr) == color_expected
-    assert (ANSI_FG_RESET in stderr) == color_expected
+    assert (__ANSI_BOLD in stderr) == color_expected
+    assert (__ANSI_FG_RED in stderr) == color_expected
+    assert (__ANSI_FG_DEFAULT in stderr) == color_expected
+    assert (__ANSI_FG_RESET in stderr) == color_expected
 
 
 def test_invalid_library(tmpdir):
@@ -784,7 +784,7 @@ typedef int MISRA_5_6_VIOLATION;
 
 # TODO: test with -j2
 # #11483
-def test_unused_function_include(tmpdir):
+def __test_unused_function_include(tmpdir, extra_args):
     test_cpp_file = os.path.join(tmpdir, 'test.cpp')
     with open(test_cpp_file, 'wt') as f:
         f.write("""
@@ -802,12 +802,31 @@ def test_unused_function_include(tmpdir):
                 };
                 """)
 
-    args = ['--enable=unusedFunction', '--inline-suppr', '--template=simple', '-j1', test_cpp_file]
+    args = [
+        '--enable=unusedFunction',
+        '--inline-suppr',
+        '--template=simple',
+        '-j1',
+        test_cpp_file
+    ]
+
+    args += extra_args
 
     _, _, stderr = cppcheck(args)
     assert stderr == "{}:4:0: style: The function 'f' is never used. [unusedFunction]\n".format(test_h_file)
 
 
+def test_unused_function_include(tmpdir):
+    __test_unused_function_include(tmpdir, [])
+
+
+# TODO: remove when we inject builddir
+def test_unused_function_include_builddir(tmpdir):
+    builddir = os.path.join(tmpdir, 'injected')
+    os.makedirs(builddir)
+    __test_unused_function_include(tmpdir, ['--cppcheck-build-dir={}'.format(builddir)])
+
+
 # TODO: test with all other types
 def test_showtime_top5_file(tmpdir):
     test_file = os.path.join(tmpdir, 'test.cpp')
@@ -1951,6 +1970,19 @@ def test_checkers_report(tmpdir):
     assert '--checkers-report' not in stderr
 
 
+def test_checkers_report_misra_json(tmpdir):
+    """check that misra checkers are reported properly when --addon=misra.json is used"""
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt') as f:
+        f.write('x=1;')
+    misra_json = os.path.join(tmpdir, 'misra.json')
+    with open(misra_json, 'wt') as f:
+        f.write('{"script":"misra.py"}')
+    exitcode, stdout, stderr = cppcheck('--enable=style --addon=misra.json --xml-version=3 test.c'.split(), cwd=tmpdir)
+    assert exitcode == 0, stdout
+    assert '<checker id="Misra C 2012: 8.1"/>' in stderr
+
+
 def test_ignore(tmpdir):
     os.mkdir(os.path.join(tmpdir, 'src'))
     test_file = os.path.join(tmpdir, 'src', 'test.cpp')
