@@ -29,7 +29,7 @@
 #include <algorithm>
 #include <list>
 #include <string>
-
+#include <vector>
 
 class TestCppcheck : public TestFixture {
 public:
@@ -59,6 +59,7 @@ private:
         TEST_CASE(unique_errors);
         TEST_CASE(isPremiumCodingStandardId);
         TEST_CASE(getDumpFileContentsRawTokens);
+        TEST_CASE(getDumpFileContentsLibrary);
     }
 
     void getErrorMessages() const {
@@ -81,14 +82,22 @@ private:
         // Check for error ids from this class.
         bool foundPurgedConfiguration = false;
         bool foundTooManyConfigs = false;
+        bool foundMissingInclude = false; // #11984
+        bool foundMissingIncludeSystem = false; // #11984
         for (const std::string & it : errorLogger.ids) {
             if (it == "purgedConfiguration")
                 foundPurgedConfiguration = true;
             else if (it == "toomanyconfigs")
                 foundTooManyConfigs = true;
+            else if (it == "missingInclude")
+                foundMissingInclude = true;
+            else if (it == "missingIncludeSystem")
+                foundMissingIncludeSystem = true;
         }
         ASSERT(foundPurgedConfiguration);
         ASSERT(foundTooManyConfigs);
+        ASSERT(foundMissingInclude);
+        ASSERT(foundMissingIncludeSystem);
     }
 
     void checkWithFile() const
@@ -102,7 +111,7 @@ private:
 
         ErrorLogger2 errorLogger;
         CppCheck cppcheck(errorLogger, false, {});
-        ASSERT_EQUALS(1, cppcheck.check(file.path()));
+        ASSERT_EQUALS(1, cppcheck.check(FileWithDetails(file.path())));
         // TODO: how to properly disable these warnings?
         errorLogger.ids.erase(std::remove_if(errorLogger.ids.begin(), errorLogger.ids.end(), [](const std::string& id) {
             return id == "logChecker";
@@ -122,8 +131,7 @@ private:
 
         ErrorLogger2 errorLogger;
         CppCheck cppcheck(errorLogger, false, {});
-        FileSettings fs;
-        fs.filename = file.path();
+        FileSettings fs{file.path()};
         ASSERT_EQUALS(1, cppcheck.check(fs));
         // TODO: how to properly disable these warnings?
         errorLogger.ids.erase(std::remove_if(errorLogger.ids.begin(), errorLogger.ids.end(), [](const std::string& id) {
@@ -147,7 +155,7 @@ private:
         const char xmldata[] = R"(<def format="2"><markup ext=".cpp" reporterrors="false"/></def>)";
         const Settings s = settingsBuilder().libraryxml(xmldata, sizeof(xmldata)).build();
         cppcheck.settings() = s;
-        ASSERT_EQUALS(0, cppcheck.check(file.path()));
+        ASSERT_EQUALS(0, cppcheck.check(FileWithDetails(file.path())));
         // TODO: how to properly disable these warnings?
         errorLogger.ids.erase(std::remove_if(errorLogger.ids.begin(), errorLogger.ids.end(), [](const std::string& id) {
             return id == "logChecker";
@@ -170,8 +178,8 @@ private:
 
         ErrorLogger2 errorLogger;
         CppCheck cppcheck(errorLogger, false, {});
-        ASSERT_EQUALS(1, cppcheck.check(test_file_a.path()));
-        ASSERT_EQUALS(1, cppcheck.check(test_file_b.path()));
+        ASSERT_EQUALS(1, cppcheck.check(FileWithDetails(test_file_a.path())));
+        ASSERT_EQUALS(1, cppcheck.check(FileWithDetails(test_file_b.path())));
         // TODO: how to properly disable these warnings?
         errorLogger.errmsgs.erase(std::remove_if(errorLogger.errmsgs.begin(), errorLogger.errmsgs.end(), [](const ErrorMessage& msg) {
             return msg.id == "logChecker";
@@ -223,6 +231,18 @@ private:
         ASSERT_EQUALS(expected, cppcheck.getDumpFileContentsRawTokens(files, tokens1));
     }
 
+    void getDumpFileContentsLibrary() const {
+        ErrorLogger2 errorLogger;
+        CppCheck cppcheck(errorLogger, false, {});
+        cppcheck.settings().libraries.emplace_back("std.cfg");
+        std::vector<std::string> files{ "/some/path/test.cpp" };
+        const std::string expected1 = "  <library lib=\"std.cfg\"/>\n";
+        ASSERT_EQUALS(expected1, cppcheck.getLibraryDumpData());
+        cppcheck.settings().libraries.emplace_back("posix.cfg");
+        const std::string expected2 = "  <library lib=\"std.cfg\"/>\n  <library lib=\"posix.cfg\"/>\n";
+        ASSERT_EQUALS(expected2, cppcheck.getLibraryDumpData());
+    }
+
     // TODO: test suppressions
     // TODO: test all with FS
 };
