@@ -20,6 +20,7 @@
 
 #include <algorithm>
 #include <array>
+#include <cstdint>
 #include <cstdlib>
 #include <fstream>
 #include <functional>
@@ -599,7 +600,7 @@ int main(int argc, char **argv)
 
     // explicit files dir..
     fout << "ifdef FILESDIR\n"
-         << "    CPPFLAGS+=-DFILESDIR=\\\"$(FILESDIR)\\\"\n"
+         << "    override CPPFLAGS+=-DFILESDIR=\\\"$(FILESDIR)\\\"\n"
          << "endif\n\n";
 
     // enable backtrac
@@ -753,7 +754,7 @@ int main(int argc, char **argv)
 
     makeConditionalVariable(fout, "PREFIX", "/usr");
     makeConditionalVariable(fout, "INCLUDE_FOR_LIB", "-Ilib -isystem externals -isystem externals/picojson -isystem externals/simplecpp -isystem externals/tinyxml2");
-    makeConditionalVariable(fout, "INCLUDE_FOR_CLI", "-Ilib -isystem externals/simplecpp -isystem externals/tinyxml2");
+    makeConditionalVariable(fout, "INCLUDE_FOR_CLI", "-Ilib -isystem externals/picojson -isystem externals/simplecpp -isystem externals/tinyxml2");
     makeConditionalVariable(fout, "INCLUDE_FOR_TEST", "-Ilib -Icli -isystem externals/simplecpp -isystem externals/tinyxml2");
 
     fout << "BIN=$(DESTDIR)$(PREFIX)/bin\n\n";
@@ -821,15 +822,6 @@ int main(int argc, char **argv)
     fout << "\t  rm -rf ${DESTDIR}${FILESDIR}; \\\n";
     fout << "\tfi\n";
     fout << "endif\n";
-    fout << "ifdef CFGDIR \n";
-    fout << "\t@if test -d ${DESTDIR}${CFGDIR}; then \\\n";
-    fout << "\t  files=\"`cd cfg 2>/dev/null && ls`\"; \\\n";
-    fout << "\t  if test -n \"$$files\"; then \\\n";
-    fout << "\t    echo '(' cd ${DESTDIR}${CFGDIR} '&&' rm -f $$files ')'; \\\n";
-    fout << "\t    ( cd ${DESTDIR}${CFGDIR} && rm -f $$files ); \\\n";
-    fout << "\t  fi; \\\n";
-    fout << "\tfi\n";
-    fout << "endif\n\n";
     fout << "# Validation of library files:\n";
     fout << "ConfigFiles := $(wildcard cfg/*.cfg)\n";
     fout << "ConfigFilesCHECKED := $(patsubst %.cfg,%.checked,$(ConfigFiles))\n";
